#ifndef INDIVIDUAL_H
#define INDIVIDUAL_H

#include "PaPreCompile.h" // QObject

class Individual : public QObject
{
  Q_OBJECT

public:
  void addChildIndividual(Individual *child); //!< Add child.
  void rewireChildren(); //!< Hand children to parent.
  void incrementChildrenAmount(); //!< Increment children amount.
  virtual QString getIndividualId() const =0    ; //!< Get individual id.
  virtual QString speciesId() const = 0; //!< Get the species id of the object.
  virtual qreal scoreSpecies() const = 0; //!< Get the score according to species rule.
  virtual qreal scoreEcoSystem() const = 0; //!< Get score according to eco system rule.
  virtual void copyInternalParameters(Individual * lastSdIndivl) = 0; //!< Copy internal parameters.
  virtual void decreaseFocus() = 0; //!< Decrease focus of mutation.
  virtual void increaseFocus() = 0; //!< Increase focus of mutaiton.
  virtual void mutate() =0; //!< Mutate the individual. Chen xin.
  virtual Individual * copy() =0; //!< Make a new copy of this individual.
  Individual *getParentIndividual() const;
  void setParentIndividual(Individual *value);
  void setSpeciesLock(bool value);
  qint32 getGeneration() const;
  qint32 getChildrenAmount() const;
  virtual ~Individual(); //!< 纯虚析构函数。

protected:
  bool speciesLock=false; //!< Whether lock species when mutating.
  qint32 generation=0; //!< generation number.

private:
  QList<Individual*> childrenIndividualList; //!< Children individual list.
  Individual * parentIndividual=nullptr; //!< The parent individual.
  qint32 childrenAmount=0; //!< Children amount.

signals:
  void scoreCalculated(const qreal & newScoreSpecies) ; //!< Signal, score calculated.
};
                                                                                                                                                      #endif
